<?php

/* -------------------------------------------------------------------------- *
 * Atom Parser PHP library                                                    *
 * Author  : 2Entwine, LLC                                                    *
 * Version : 0.1                                                              *
 * Created : 2004.10.23                                                       *
 * Homepage: http://fotobuzz.org/                                             *
 * License : LGPL License                                                     * 
 * -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- *
 * Namespace functions                                                        *
 * -------------------------------------------------------------------------- */

function NS($namespace, $elementName) {
  if ($elementName != NULL)
    return "$namespace:$elementName";
  else
    return $namespace;
}

function AtomNS($elementName = NULL) {
  return NS('http://purl.org/atom/ns#', $elementName);
}

function DCTermsNS($elementName = NULL) {
  return NS('http://purl.org/dc/terms/', $elementName);
}

function FotoBuzzNS($elementName = NULL) {
#  if (func_num_args() == 0)
#    $elementName = NULL;
#  else
#    $elementName = func_get_arg(0);

  return NS('http://fotobuzz.org/fb/ns#', $elementName);
}

/* -------------------------------------------------------------------------- *
 * AtomParser class                                                           *
 * -------------------------------------------------------------------------- */

class AtomParser {

  /* ------------------------------------------------------------------------ *
   * Public methods                                                           *
   * ------------------------------------------------------------------------ */

  function AtomParser() {

    $this->_parser = xml_parser_create_ns();
    xml_parser_set_option($this->_parser, XML_OPTION_CASE_FOLDING, 0);
    xml_set_object($this->_parser, $this);
    
    # Handler initialization
    $this->_handlerStack = array();
    $this->_currentHandlerSet = NULL;
    $this->_pushHandlerStack($this->FEED_HANDLER_SET);

    $this->_currentElementName = NULL;

    $this->_feedDict = array('entries' => array(), 
			     'links' => array(),
			     'authors' => array());
    $this->_entryDict = NULL;
    $this->_authorDict = NULL;
  }

  function parse($xml) {
    $status = xml_parse($this->_parser, $xml);
  }

  function getFeed() {
    return $this->_feedDict;
  }

  /* ------------------------------------------------------------------------ *
   * Non-public methods below                                                 *
   * ------------------------------------------------------------------------ */

  function _pushHandlerStack($handlerSet) {
    # Changes the set of the handler functions currently used by the Expat
    # parser. It pushes the current set of handlers onto a stack to 
    # restored later.

    list($startHandlerFunc, $endHandlerFunc, $cdataHandlerFunc) = $handlerSet;

    array_push($this->_handlerStack, $this->_currentHandlerSet);
    $this->_currentHandlerSet = $handlerSet;

    xml_set_element_handler($this->_parser, 
			    $startHandlerFunc,
			    $endHandlerFunc);
    xml_set_character_data_handler($this->_parser, $cdataHandlerFunc);
  }

  function _popHandlerStack() {
    # Restores the set of handlers at the top of the handler stack.

    $this->_currentHandlerSet = array_pop($this->_handlerStack);
 
    xml_set_element_handler($this->_parser,
			    $this->_currentHandlerSet[0],
			    $this->_currentHandlerSet[1]);
    xml_set_character_data_handler($this->_parser, $this->_currentHandlerSet[2]);
  }

  function _splitQName($QName) {

    preg_match("/(.+):([a-zA-Z]+)/", $QName, $matches);
    if ($matches) {
      return array($matches[1], $matches[2]);
    }
    else {
      return array(NULL, $QName);
    }
  }

  /* ------------------------------------------------------------------------ *
   * Feed Level Handlers                                                      *
   * ------------------------------------------------------------------------ */

  var $FEED_HANDLER_SET = array('_feedElementStart', 
				'_feedElementEnd', 
				'_feedCData');

  function _feedElementStart($parser, $QName, $attributes) {
    list($namespace, $elementName) = $this->_splitQName($QName);

    if ($QName == AtomNS('entry')) {
      # Found a new entry. Make room for it and add to it the feed structure.
      unset($this->_entryDict);
      $this->_entryDict = array('authors'=> array(), 'links' => array());
      array_push($this->_feedDict['entries'], &$this->_entryDict);

      # Switch the entry handlers to parse the Atom entry.
      $this->_pushHandlerStack($this->ENTRY_HANDLER_SET);
    }
    else if ($QName == AtomNS('author')) {
      # Found an author. Allocate a dict for the author and add it to the 
      # feed structure.
      unset($this->_authorDict);
      $this->_authorDict = array();
      array_push($this->_feedDict['authors'], &$this->_authorDict);

      # Switch the author handlers to parse the Atom author XML.
      $this->_pushHandlerStack($this->AUTHOR_HANDLER_SET);
    }
    else if ($QName == AtomNS('link')) {
      array_push($this->_feedDict["links"], $attributes);
    }
    else if ($namespace == AtomNS()) {
      $this->_feedDict[$elementName] = "";
      $this->_currentElementName = $elementName;
    }
  }

  function _feedElementEnd($parser, $QName) {
    list($namespace, $elementName) = $this->_splitQName($QName);

    if ($namespace == AtomNS()) {
      # Delete the item from feedDict if the item's value if empty.
      if (array_key_exists($elementName, $this->_feedDict) and
	  $this->_feedDict[$elementName] == "") {
	unset($this->_feedDict[$elementName]);
      }

      $this->_currentElementName = NULL;
    }
  }

  function _feedCData($parser, $cdata) {

    if ($this->_currentElementName) {
      $this->_feedDict[$this->_currentElementName] .= $cdata;
    }
  }

  /* ------------------------------------------------------------------------ *
   * Entry Level Handlers                                                     *
   * ------------------------------------------------------------------------ */

  var $ENTRY_HANDLER_SET = array('_entryElementStart', 
				 '_entryElementEnd', 
				 '_entryCData');

  function _entryElementStart($parser, $QName, $attributes) {
    list($namespace, $elementName) = $this->_splitQName($QName);

    if ($QName == AtomNS('author')) {
      unset($this->_authorDict);
      $this->_authorDict = array("name" => "");
      array_push($this->_entryDict['authors'], &$this->_authorDict);

      $this->_pushHandlerStack($this->AUTHOR_HANDLER_SET);
    }
    else if ($QName == AtomNS('link')) {
      
    }
    else if ($namespace == AtomNS()) {
      $this->_currentElementName = $elementName;
      $this->_entryDict[$this->_currentElementName] = "";
    }
    else if ($QName == FotoBuzzNS("boundingBox")) {
      $values = array($attributes['x'], $attributes['y'], 
		      $attributes['width'], $attributes['height']);

      $this->_entryDict['boundingBox'] = $values;
    }
    else if ($Qname == AtomNS("content")) {

      if (array_key_exists('type', $attributes))
	$this->_contentType = $attributes['type'];
      else
	$this->_contentType = "text/plain";

      if (array_key_exists('mode', $attributes))
	$this->_contentMode = $attributes['mode'];
      else
	$this->_contentMode = "xml";

      # Only handle the following content types
      if ('text/plain' != $this->_contentType)
	continue;

      $this->_entryDict['content'] = "";
      $this->_pushHandlerStack($this->CONTENT_HANDLER_SET);
    }
  }

  function _entryElementEnd($parser, $QName) {
    list($namespace, $elementName) = $this->_splitQName($QName);

    if ($QName == AtomNS('entry')) {
      # Remove properties from an entry if they're empty
      foreach(array_keys($this->_entryDict) as $key) {
	if (!$this->_entryDict[$key])
	  unset($this->_entryDict[$key]);
      }
      foreach(array('authors', 'links') as $key) {
	if (!count($this->_entryDict[$key])) {
	  unset($this->_entryDict[$key]);
	}
      }

      $this->_popHandlerStack();
    }
    else if ($namespace == AtomNS()) {
      $this->_currentElementName = NULL;
    }
  }

  function _entryCData($parser, $cdata) {
    if ($this->_currentElementName) {
      $this->_entryDict[$this->_currentElementName] .= $cdata;
    }
  }

  /* ------------------------------------------------------------------------ *
   * Author Level Handlers                                                    *
   * ------------------------------------------------------------------------ */

  var $AUTHOR_HANDLER_SET = array('_authorElementStart',
				  '_authorElementEnd',
				  '_authorCData');

  function _authorElementStart($parser, $QName, $attributes) {
    list($namespace, $elementName) = $this->_splitQName($QName);

    if ($namespace == AtomNS()) {
      $this->_currentElementName = $elementName;
      $this->_authorDict[$elementName] = "";
    }
  }

  function _authorElementEnd($parser, $QName) {
    if ($QName == AtomNS('author')) {
      $this->_popHandlerStack();
    }
  }

  function _authorCData($parser, $cdata) {
    if ($this->_currentElementName) {
      $this->_authorDict[$this->_currentElementName] .= $cdata;
    }
  }


  /* ------------------------------------------------------------------------ *
   * Content Level Handlers                                                   *
   * ------------------------------------------------------------------------ */

  var $CONTENT_HANDLER_SET = array('_contentElementStart',
				   '_contentElementEnd',
				   '_contentCData');

  function _contentElementStart($parser, $QName, $attributes) {
    list($namespace, $elementName) = $this->_splitQName($QName);

    $xml = "<$elementName";
    
    foreach($attributes as $key => $value) {
      $xml .= " $key='$value'";
    }
      
    $xml = ">";
    $this->_entryDict['content'] .= $xml;
  }

  function _contentElementEnd($parser, $QName) {

    if ($QName == AtomNS('content')) {
      # Need to decode the base64 since we're done collecting all the CDATA
      # comprising the the base64 content.
      $this->_entryDict['content'] .= base64_decode($this->_entryDict['content']);

      $this->_popHandlerStack();
      return;
    }

    list($namespace, $elementName) = $this->_splitQName($QName);

    $xml = "<$elementName/>";
    $this->_entryDict['content'] .= $xml;
  }

  function _contentCData($parser, $cdata) {
    $cdata = stripslashes($cdata);

    if ($this->_contentMode == 'escaped')
      $this->_entryDict['content'] .= xmlUnescape($cdata);
    else
      $this->_entryDict['content'] .= $cdata;
  }
}

/* -------------------------------------------------------------------------- *
 * XML Utility functions                                                      *
 * -------------------------------------------------------------------------- */

function xmlEscape($text) {
  $patterns[0] = "'&'";
  $patterns[1] = "'<'";
  $patterns[2] = "'>'";
  $patterns[3] = "'\''";
  $patterns[4] = "'\"'";

  $replacements[0] = "&amp;";
  $replacements[1] = "&lt;";
  $replacements[2] = "&gt;";
  $replacements[3] = "&apos;";
  $replacements[4] = "&quot;";

  return preg_replace($patterns, $replacements, $text);
}

function xmlUnescape($text) {
  $patterns[0] = "'&amp;'";
  $patterns[1] = "'&lt;'";
  $patterns[2] = "'&gt;'";
  $patterns[3] = "'&apos;'";
  $patterns[4] = "'&quot;'";
  
  $replacements[0] = "&";
  $replacements[1] = "<";
  $replacements[2] = ">";
  $replacements[3] = "'";
  $replacements[4] = "\"";

  return preg_replace($patterns, $replacements, $text);
}


/* -------------------------------------------------------------------------- *
 * Tag URI generator                                                          *
 * -------------------------------------------------------------------------- */

function generateTagURI($authority, $specific) {
  $wide = 100000000;

  if ($specific == NULL)
    $specific = rand(0, $wide);


  $isodate = strftime("%Y-%m-%d");
  $tagURI = "tag:$authority,$isodate:$specific";

  return $tagURI;
}

?>