<?php

/* -------------------------------------------------------------------------- *
 * FotoBuzz JFIF Header Parsing PHP library                                   *
 * Author  : 2Entwine, LLC                                                    *
 * Version : 0.1                                                              *
 * Created : 2004.10.23                                                       *
 * Homepage: http://fotobuzz.org/                                             *
 * License : LGPL License                                                     * 
 * -------------------------------------------------------------------------- */

class FBJFIF {

  var $JFIF_MARKER = 0xd8;
  var $FOTOBUZZ_MARKER = 0xe8;
  var $APP0_MARKER = 0xe0;

  function readFotonotesData($fileObj) {
    $data = NULL;
    list($start, $end) = $this->_findMarker($fileObj, $this->FOTOBUZZ_MARKER);

    if ($start != NULL and $end != NULL) {
      fseek($fileObj, $start + 4);
      $data = fread($fileObj, $end - $start - 4);
    }

    return $data;
  }

  function createJPEG($fileObj, $xml) {
    $jpeg = "";
    list($start, $end) = $this->_findMarker($fileObj, $this->FOTOBUZZ_MARKER);

    if ($start and $end) {
      # Replace pre-existing Fotonotes data.
      fseek($fileObj, 0);
      $jpeg .= fread($fileObj, $start);
      $jpeg .= pack("CCn", 0xff, $this->FOTOBUZZ_MARKER, strlen($xml) + 2);
      $jpeg .= $xml;

      fseek($fileObj, $end);
      $size = $this->_fileSize($fileObj);
      $jpeg .= fread($fileObj, $size);
    }
    else {
      # No pre-existing Fotonotes data, so insert a new header.
      list($start, $end) = $this->_findMarker($fileObj, $this->APP0_MARKER);

      fseek($fileObj, 0);
      $jpeg .= fread($fileObj, $end);
      $jpeg .= pack("CCn", 0xff, $this->FOTOBUZZ_MARKER, strlen($xml) + 2);
      $jpeg .= $xml;

      # Find the size of the file
      $size = $this->_fileSize($fileObj);
      $jpeg .= fread($fileObj, $size);
    }

    return $jpeg;
  }

  function _fileSize($fileObj) {
    $marker = ftell($fileObj);
    fseek($fileObj, 0, SEEK_END);
    $size = ftell($fileObj);
    fseek($fileObj, $marker);

    return $size;
  }

  function _readMarker($fileObj) {
    if (hexdec(bin2hex(fread($fileObj, 1))) == 0xff) {
      $bytes = unpack("C", fread($fileObj, 1));
      return $bytes[1];
    }

    return NULL;
  }

  function _readHeader($fileObj) {
    $marker = $this->_readMarker($fileObj);
    if ($marker != NULL) {
      $bytes = unpack('n',fread($fileObj, 2));
      $headerSize = $bytes[1] - 2;
      return array($marker,  $headerSize);
    }
    else {
      return array(NULL, NULL);
    }
  }

  function _findMarker($fileObj, $markerType) {
    fseek($fileObj, 0);

    $startOffset = NULL;
    $endOffset = NULL;

    # look for SOI marker
    if ($this->_readMarker($fileObj) == $this->JFIF_MARKER) {
      list($headerType, $headerSize) = $this->_readHeader($fileObj);

      while ($headerType != NULL) {
	if($headerType == $markerType) {
	  $currentFileOffset = ftell($fileObj);
	  $startOffset = $currentFileOffset - 4;
	  $endOffset = $currentFileOffset + $headerSize;
	  break;
	}
	else {
	  fseek($fileObj, ftell($fileObj) + $headerSize);
	}

	list($headerType, $headerSize) = $this->_readHeader($fileObj);
      }
    }

    return array($startOffset, $endOffset);
  }
}

?>
