<?php

ini_set("log_errors", True);

/* -------------------------------------------------------------------------- *
 * Fotobuzz PHP script                                                        *
 * Author  : 2Entwine, LLC                                                    *
 * Version : 0.1                                                              *
 * Created : 2004.10.23                                                       *
 * Homepage: http://fotobuzz.org/                                             *
 * License : LGPL License                                                     * 
 * -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- *
 * User Configuration                                                         *
 * -------------------------------------------------------------------------- */

# Either your domain name or your email address. ID_AUTHORITY is used
# for generating ids for annotation entries.
# (Required to be changed)
$ID_AUTHORITY = "localhost";

# The copyright to be interested into the picture's feed
$DEFAULT_COPYRIGHT = "Copyright, 2004";

# This script will only read annotations but won't allow writing of
# annotations back into the JPEG.
$ALLOW_NEW_ANNOTATIONS = True;

# Only allow annotations to be added which means that modifying and deleting
# annotations isn't allowed. $ALLOW_NEW_ANNOTATIONS has to be true for this
# parameter to matter.
$ALLOW_ONLY_ADD_ANNOTATIONS = False;

# Restrict the script to only modify pictures below the specified
# directory.
# eg. Windows: C:/Program Files/Apache Group/Apache2/htdocs/fotobuzz/
# eg. Unix:    /usr/local/www/fotobuzz/
$IMAGES_ROOT = NULL;

/* -------------------------------------------------------------------------- *
 * Content Generation                                                         *
 * -------------------------------------------------------------------------- */

include_once 'fbjfif.inc';
include_once 'atomparser.inc';
include_once 'fotobuzzxml.inc';
include_once 'utility.inc';

if ($_SERVER['REQUEST_METHOD'] == 'GET' && isset($_GET['image'])) {
  # Read Fotonotes data from image.
  $docRoot = $_SERVER['DOCUMENT_ROOT'];
  $fileName = $docRoot . $_GET['image'];
  $fileName = realpath($fileName);

  # Check to make sure that the requested file is in a directory viewable
  # to users of this script.
  if ($IMAGES_ROOT) {
    $restrictedBase = realpath($IMAGES_ROOT);
    if (! $restrictedBase) {
      echo "\$IMAGES_ROOT path doesn't exist. Please double check value";
      return;
    }

    if (strncmp($restrictedBase, $fileName, strlen($restrictedBase))) {
      header("HTTP/1.1 404 Not Found");
      echo "Image could not be found. Verify the path being passed to the image parameter";
      return;
    }
  }

  if (! file_exists($fileName)) {
    header("HTTP/1.1 404 Not Found");
    echo "Image could not be found. Verify the path being passed to the image parameter";
    return;
  }

  $jpegParser = new FBJFIF;
  $jpegFileObj = fopen($fileName, "rb");

  # Get a shared lock on the file
  $xml = $jpegParser->readFotonotesData($jpegFileObj);

  $parser = new AtomParser();
  $parser->parse($xml);
  $atomDict = $parser->getFeed();

  $atomDict['entries'] = removeBadEntries($atomDict['entries']);

  // and print the resulting XML
  header("Content-type: text/xml");
  echo fotobuzzStructToXML($atomDict, 
			   $ALLOW_NEW_ANNOTATIONS, 
			   $ALLOW_ONLY_ADD_ANNOTATIONS);

  fclose($jpegFileObj);
}
else if ($ALLOW_NEW_ANNOTATIONS && $_SERVER['REQUEST_METHOD'] == 'POST') {
  # Write Fotonotes update to image.
  $docRoot = $_SERVER['DOCUMENT_ROOT'];
  $fileName = $docRoot . $_POST['image'];
  $fileName = realpath($fileName);

  # Check to make sure that the requested file is in a directory viewable
  # to users of this script.
  if ($IMAGES_ROOT) {
    $restrictedBase = realpath($IMAGES_ROOT);
    if (! $restrictedBase) {
      echo "\$IMAGES_ROOT path doesn't exist. Please double check value";
      return;
    }

    if (strncmp($restrictedBase, $fileName, strlen($restrictedBase))) {
      header("HTTP/1.1 404 Not Found");
      echo "Image could not be found. Verify the path being passed to the image parameter";
      return;
    }
  }

  if (! file_exists($fileName)) {
    header("HTTP/1.1 404 Not Found");
    echo "Image could not be found. Verify the path being passed to the image parameter";
    return;
  }

  $xml = stripslashes($_POST['xml']);

  # Read the original fotonotes data
  $jpegParser = new FBJFIF;

  if (lockFile($fileName)) {
    $jpegFileObj = fopen($fileName, "r+b");
    $fotonotesXML = $jpegParser->readFotonotesData($jpegFileObj);

    $parser = new AtomParser;
    $parser->parse($fotonotesXML);
    $atomDict = $parser->getFeed();

    # Parse the received xml
    $receivedDict = fotobuzzXMLToStruct($xml);

    # Merge the existing fotonotes data with the new fotonotes data.
    $newDict = fotobuzzStructMerge($receivedDict, 
				   $atomDict, 
				   $ALLOW_ONLY_ADD_ANNOTATIONS,
				   $ID_AUTHORITY, 
				   $DEFAULT_COPYRIGHT);

    # Remove bad entries before writing the XML back into the JPEG.
    $newDict['entries'] = removeBadEntries($newDict['entries']);
    $newXML = fotobuzzStructToAtom($newDict);

    $newJPEGData = $jpegParser->createJPEG($jpegFileObj, $newXML);
    fseek($jpegFileObj, 0);
    fwrite($jpegFileObj, $newJPEGData);
  
    fclose($jpegFileObj);
    unlockFile($fileName);

    # Success. Flash is "ignant" of HTTP status codes so
    # this is to signal that the write occurred.
    header("Content-type: text/plain");
    echo "ok";
  }
  else {
    header("Content-type: text/plain");
    echo "Could not lock file to write annotations to it.";
  }
}

?>
